package com.wowza.wms.plugin.streamnamealias;

import java.util.*;
import java.io.*;

import com.wowza.util.*;
import com.wowza.wms.module.*;
import com.wowza.wms.application.*;
import com.wowza.wms.stream.*;

public class ModuleStreamNameAlias extends ModuleBase implements IMediaStreamNameAliasProvider
{	
	public static final String STREAMALIASFILE_PROPERTY = "aliasMapFileStream";
	public static final String PLAYALIASFILE_PROPERTY = "aliasMapFilePlay";
	public static final String STREAMALIASDEBUG_PROPERTY = "aliasMapDebug";
	public static final String STREAMALIASPATHDELIMITER_PROPERTY = "aliasMapPathDelimiter";
	public static final String STREAMALIASNAMEDELIMITER_PROPERTY = "aliasMapNameDelimiter";
	
	private Map<String, String> envMap = new HashMap<String, String>();
	private File streamFilePtr = null;
	private File playFilePtr = null;
	private boolean debug = false;
	private String pathDelimiter = "/";
	private String nameDelimiter = "=";

	public void onAppStart(IApplicationInstance appInstance)
	{
		WMSProperties props = appInstance.getProperties();
		debug = props.getPropertyBoolean(STREAMALIASDEBUG_PROPERTY, debug);
		pathDelimiter = props.getPropertyStr(STREAMALIASPATHDELIMITER_PROPERTY, pathDelimiter);
		nameDelimiter = props.getPropertyStr(STREAMALIASNAMEDELIMITER_PROPERTY, nameDelimiter);
		
		StreamNameAliasFiles.setNameDelimeter(nameDelimiter);

		Map pathMap = new HashMap();
		pathMap.put("com.wowza.wms.context.VHost", appInstance.getVHost().getName());
		pathMap.put("com.wowza.wms.context.VHostConfigHome", appInstance.getVHost().getHomePath());
		pathMap.put("com.wowza.wms.context.Application", appInstance.getApplication().getName());
		pathMap.put("com.wowza.wms.context.ApplicationInstance", appInstance.getName());

		while(true)
		{
			String aliasPath = props.getPropertyStr(STREAMALIASFILE_PROPERTY);
			if (aliasPath == null)
			{
				getLogger().warn("ModuleStreamNameAlias.onAppStart: Property "+STREAMALIASFILE_PROPERTY+" is missing. Can't find alias file.");
				break;
			}

			aliasPath =  SystemUtils.expandEnvironmentVariables(aliasPath, pathMap);
			this.streamFilePtr = new File(aliasPath);
			
			if (debug)
				getLogger().info("ModuleStreamNameAlias.onAppStart: "+STREAMALIASFILE_PROPERTY+": "+streamFilePtr.getAbsolutePath());
			break;
		}
		
		while(true)
		{
			String aliasPath = props.getPropertyStr(PLAYALIASFILE_PROPERTY);
			if (aliasPath == null)
			{
				getLogger().warn("ModuleStreamNameAlias.onAppStart: Property "+PLAYALIASFILE_PROPERTY+" is missing. Can't find alias file.");
				break;
			}

			aliasPath =  SystemUtils.expandEnvironmentVariables(aliasPath, pathMap);
			this.playFilePtr = new File(aliasPath);
			
			if (debug)
				getLogger().info("ModuleStreamNameAlias.onAppStart: "+PLAYALIASFILE_PROPERTY+": "+playFilePtr.getAbsolutePath());
			break;
		}
		
		envMap.put("VHost.Name", appInstance.getVHost().getName());
		envMap.put("Application.Name", appInstance.getApplication().getName());
		envMap.put("ApplicationInstance.Name", appInstance.getName());
		envMap.put("AppInstance.Name", appInstance.getName());
		
		appInstance.setStreamNameAliasProvider(this);
	}
	
	public void onAppStop(IApplicationInstance appInstance)
	{
		getLogger().info("ModuleStreamNameAlias.onAppStop: "+appInstance.getApplication().getName()+"/"+appInstance.getName());
	}
	
	private String nameToAlias(String aliasType, File filePtr, String streamName)
	{
		String ret = null;

		while(true)
		{
			if (filePtr == null)
			{
				if (debug)
					getLogger().info("ModuleStreamNameAlias.nameToAlias["+aliasType+"]: filePtr is null");
				break;
			}
			
			StreamNameAliasFiles aliasFiles = StreamNameAliasFiles.getInstance();
			if (aliasFiles == null)
			{
				if (debug)
					getLogger().info("ModuleStreamNameAlias.nameToAlias["+aliasType+"]: aliasFiles is null");
				break;
			}
			
			StreamNameAliasFile aliasFile = aliasFiles.getMapFile(filePtr);
			if (aliasFile == null)
			{
				if (debug)
					getLogger().info("ModuleStreamNameAlias.nameToAlias["+aliasType+"]: aliasFile missing: "+filePtr.getAbsolutePath());
				break;
			}
			
			//TODO - performance - add caching mechanism
			
			StreamNameAliasMatch aliasMatch = aliasFile.findMap(streamName);
			if (aliasMatch == null)
			{
				if (debug)
					getLogger().info("ModuleStreamNameAlias.nameToAlias["+aliasType+"]: Can't find alias entry for: "+streamName);
				break;
			}
			
			Map<String, String> sendMap = new HashMap<String, String>();
			sendMap.putAll(envMap);
			
			sendMap.put("Stream.Name", streamName);
			if (streamName.indexOf(pathDelimiter) >= 0)
			{
				String[] parts = streamName.split("["+pathDelimiter+"]");
				for(int i=0;i<parts.length;i++)
					sendMap.put("Stream.Name.Part"+(i+1), parts[i]);
			}
			else
				sendMap.put("Stream.Name.Part1", streamName);
			
			if (aliasMatch.wildcardMatches != null)
			{
				int gindex = 0;
				Iterator<String> iter = aliasMatch.wildcardMatches.iterator();
				while(iter.hasNext())
				{
					String value = iter.next();
					sendMap.put("Wildcard.Match"+(gindex+1), value);
					gindex++;
				}
			}
			
			ret =  SystemUtils.expandEnvironmentVariables(aliasMatch.alias, sendMap);
			if (debug)
				getLogger().info("ModuleStreamNameAlias.nameToAlias["+aliasType+"]: streamName:"+streamName+" alias:"+aliasMatch+" result:"+ret);
			break;
		}

		return ret;
	}
	
	public String resolvePlayAlias(IApplicationInstance appInstance, String name)
	{
		return nameToAlias("play", playFilePtr, name);
	}
	public String resolveStreamAlias(IApplicationInstance appInstance, String name)
	{
		return nameToAlias("stream", streamFilePtr, name);
	}
}
