package com.wowza.wms.plugin.streamnamealias;

import java.io.*;
import java.util.*;
import java.util.regex.*;

import com.wowza.wms.logging.*;

public class StreamNameAliasFile
{
	class StreamAlias
	{
		String pattern = null;
		String alias = null;
		
		public StreamAlias(String pattern, String alias)
		{
			this.pattern = pattern;
			this.alias = alias;
		}
		
		public String toString()
		{
			return "{pattern: \""+pattern+"\" alias: \""+alias+"\"}";
		}
	}

	private long lastModDate = -1;
	private long lastSize = -1;
	private File file = null;
	private List<StreamAlias> aliasVals = new ArrayList<StreamAlias>();
	private Map<String, StreamAlias> aliasMap = new HashMap<String, StreamAlias>();
	private boolean aliasMapAllNonWild = true;
	private boolean aliasMapLastMatchAll = false;
	private long lastReloadChecktime = -1;
	
	
	public StreamNameAliasFile(File file)
	{
		this.file = file;
		load();
	}
	
	private void load()
	{
		WMSLoggerFactory.getLogger(StreamNameAliasFile.class).debug("StreamNameAliasFile.load: "+file.getAbsolutePath());
		aliasVals.clear();
		aliasMap.clear();
		aliasMapAllNonWild = true;
		aliasMapLastMatchAll = false;
		
		String delimter = StreamNameAliasFiles.getNameDelimeter();
		
		BufferedReader inf = null;
		try
		{
			inf = new BufferedReader(new FileReader(file));
			String line;
			int wildCount = 0;
			while ((line = inf.readLine()) != null)
			{
				line = line.trim();
				if (line.startsWith("#"))
					continue;
				if (line.length() == 0)
					continue;
				
				String pattern = null;
				String alias = null;
				int pos = line.indexOf(delimter);
				if (pos >= 0)
				{
					pattern = line.substring(0, pos).trim();
					alias = line.substring(pos+1).trim();
				}
				if (pattern != null && alias != null)
				{
					StreamAlias streamAlias = new StreamAlias(pattern, alias);
					aliasVals.add(streamAlias);
					boolean isWild = pattern.indexOf("*") >= 0;
					if (!isWild)
						aliasMap.put(pattern, streamAlias);
					else
						wildCount++;
				}
				
			}
			
			if (aliasVals.size() > 0)
			{
				StreamAlias streamAlias = aliasVals.get(aliasVals.size()-1);
				if (streamAlias.pattern.equals("*"))
				{
					aliasMapLastMatchAll = true;
					wildCount--;
				}
			}
			
			if (wildCount > 0)
			{
				aliasMap.clear();
				aliasMapAllNonWild = false;
			}
			
			WMSLoggerFactory.getLogger(StreamNameAliasFile.class).info("StreamNameAliasFile.load["+file.getAbsolutePath()+"]: size:"+aliasMap.size()+":"+aliasVals.size()+" hash:"+aliasMapAllNonWild+" matchAll:"+aliasMapLastMatchAll);
		}
		catch (Exception e)
		{
			WMSLoggerFactory.getLogger(StreamNameAliasFile.class).error("StreamNameAliasFile.load: "+e.toString());
		}
		
		try
		{
			if (inf != null)
				inf.close();
			inf = null;
		}
		catch (Exception e)
		{
			WMSLoggerFactory.getLogger(StreamNameAliasFile.class).error("StreamNameAliasFile.load[close]: "+e.toString());
		}
		
		lastModDate = file.lastModified();
		lastSize = file.length();
	}
	
	private void checkReload()
	{
		if (needReload())
			load();
	}
	
	public boolean needReload()
	{
		long currTime = System.currentTimeMillis();
		if (lastReloadChecktime < 0 || (currTime-lastReloadChecktime) > 2000)
		{
			lastReloadChecktime = currTime;
			return (lastModDate != file.lastModified() || lastSize != file.length());
		}
		
		return false;
	}
	
	private StreamNameAliasMatch patternMatch(String streamName, StreamAlias streamAlias)
	{
		StreamNameAliasMatch ret = null;

		String pattern = streamAlias.pattern;
		if (pattern.equals("*"))
		{
			ret = new StreamNameAliasMatch();
			
			ret.pattern = streamAlias.pattern;
			ret.alias = streamAlias.alias;
			ret.wildcardMatches = new ArrayList<String>();
			ret.wildcardMatches.add(streamName);
		}
		else
		{
			String regPattern = pattern;
			regPattern = regPattern.replace(".", "\\.");
			regPattern = regPattern.replace("*", "(.*)");
			regPattern = "^"+regPattern+"$";
			Pattern p = Pattern.compile(regPattern);
			Matcher m = p.matcher(streamName);
			if (m.find())
			{
				boolean isMatch = (m.start() == 0 && m.end() == streamName.length());
				if (isMatch)
				{
					ret = new StreamNameAliasMatch();
					
					ret.pattern = streamAlias.pattern;
					ret.alias = streamAlias.alias;
	
					if (m.groupCount() > 0)
					{
						ret.wildcardMatches = new ArrayList<String>();
						for(int i=0;i<m.groupCount();i++)
							ret.wildcardMatches.add(m.group(i+1));
					}
				}
			}
		}

		return ret;
	}
		
	public synchronized StreamNameAliasMatch findMap(String streamName)
	{
		checkReload();
		
		StreamNameAliasMatch ret = null;
		
		//System.out.println("find: "+streamName);
		if (aliasMapAllNonWild)
		{
			StreamAlias streamAlias = aliasMap.get(streamName.trim());
			if (streamAlias != null)
			{
				ret = new StreamNameAliasMatch();
				ret.pattern = streamAlias.pattern;
				ret.alias = streamAlias.alias;
				//System.out.println(" found[get]: "+ret.pattern+":"+ret.alias);
			}
			else if (aliasMapLastMatchAll)
			{
				ret = patternMatch(streamName, aliasVals.get(aliasVals.size()-1));
				//if (ret != null)
				//{
				//	System.out.println(" found[match1]: "+ret.pattern+":"+ret.alias);
				//}
			}
		}
		else
		{
			Iterator<StreamAlias> iter = aliasVals.iterator();
			while(iter.hasNext())
			{
				StreamAlias streamAlias = iter.next();
				String pattern = streamAlias.pattern;
				if (pattern.indexOf("*") >= 0)
				{
					ret = patternMatch(streamName, streamAlias);
					if (ret != null)
					{
						//System.out.println(" found[match2]: "+ret.pattern+":"+ret.alias);
						break;
					}
				}
				else if (pattern.equals(streamName))
				{
					ret = new StreamNameAliasMatch();
					ret.pattern = streamAlias.pattern;
					ret.alias = streamAlias.alias;
					//System.out.println(" found[equals]: "+ret.pattern+":"+ret.alias);
					break;
				}
			}
		}

		return ret;
	}
}
